/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Values.cpp
 *
 *  Created on: Feb 10, 2011
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#include "CodePatterns/MemDebug.hpp"

#include "CodePatterns/Assert.hpp"

#include <boost/lexical_cast.hpp>
#include <boost/tokenizer.hpp>

#include "Box.hpp"
#include "LinearAlgebra/BoxVector.hpp"
#include "LinearAlgebra/RealSpaceMatrix.hpp"
#include "LinearAlgebra/Vector.hpp"

#include "Values.hpp"

static const Vector parseAsVector(const std::string &_string)
{
  Vector temp;
  // dissect by ","
  typedef boost::tokenizer<boost::char_separator<char> > tokenizer;
  boost::char_separator<char> value_separator(",)(");

  bool status = true;
  tokenizer tokens(_string, value_separator);
  if (!_string.empty()) {
    tokenizer::iterator tok_iter = tokens.begin();
    for (size_t i=0;i<NDIM;++i) {
      if (tok_iter == tokens.end()) {
        status = false;
        break;
      }
      temp[i] = boost::lexical_cast<double>(*(tok_iter++));
    }
  }
  if (!status)
    temp.Zero();

  return temp;
}

Vector VectorValue::toVector() const
{
  Vector returnVector = parseAsVector(vectorstring);

  return returnVector;
}

BoxVector VectorValue::toBoxVector(Box &_box) const
{
  BoxVector returnVector;
  static_cast<Vector>(returnVector) = parseAsVector(vectorstring); // under its hood it's still a Vector

  ASSERT(_box.isValid(returnVector),
      "VectorValue::toBoxVector() - vector "+toString(toVector())+" does not lie within box "+toString(_box)+".");

  return returnVector;
}

RealSpaceMatrix RealSpaceMatrixValue::toRealSpaceMatrix() const
{
  RealSpaceMatrix returnMatrix(ReturnFullMatrixforSymmetric(matrix));

  return returnMatrix;
}
